import cv2
import math
from gpiozero import LED
led = LED(17)
def calculate_distance(point1, point2):
    return math.sqrt((point2[0] - point1[0]) ** 2 + (point2[1] - point1[1]) ** 2)

def main():
    cap = cv2.VideoCapture(0)  # Open the webcam (change the index if using a different camera)

    while True:
        ret, frame = cap.read()
        if not ret:
            break

        # Convert the frame to grayscale
        gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)

        # Apply a Gaussian blur to reduce noise
        blurred = cv2.GaussianBlur(gray, (7, 7), 0)

        # Perform edge detection
        edges = cv2.Canny(blurred, 50, 150)

        # Find contours of the hand
        contours, _ = cv2.findContours(edges, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)

        # Find the hand contour with the largest area
        hand_contour = max(contours, key=cv2.contourArea)

        # Draw the hand contour
        cv2.drawContours(frame, [hand_contour], 0, (0, 255, 0), 2)

        # Find the convex hull of the hand contour
        hull = cv2.convexHull(hand_contour)

        # Draw the convex hull
        cv2.drawContours(frame, [hull], 0, (0, 0, 255), 2)

        # Find the defects in the convex hull
        defects = cv2.convexityDefects(hand_contour, cv2.convexHull(hand_contour, returnPoints=False))

        if defects is not None:
            for i in range(defects.shape[0]):
                s, e, f, _ = defects[i, 0]
                start = tuple(hand_contour[s][0])
                end = tuple(hand_contour[e][0])
                far = tuple(hand_contour[f][0])

                # Draw the finger skeleton lines
                cv2.line(frame, start, end, (255, 0, 0), 2)
                cv2.circle(frame, far, 5, (0, 255, 0), -1)

                # Calculate the distance between the thumb and index finger tips
                thumb_tip = start
                index_tip = end
                distance = calculate_distance(thumb_tip, index_tip)

                # Print the distance
                cv2.putText(frame, f"Distance: {distance:.2f} pixels", (50, 50), cv2.FONT_HERSHEY_SIMPLEX, 1,
                            (0, 0, 255), 2)

                # Check if the thumb touches the index finger tip
                if distance < 10:
                    cv2.putText(frame, "Thumb touching index finger tip", (50, 100), cv2.FONT_HERSHEY_SIMPLEX, 1,
                                (0, 0, 255), 2)
                    led.on()
                else:
                    cv2.putText(frame, "Lights Off", (50, 100), cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 0, 255), 2)
                    led.off()

        cv2.imshow("Hand Tracking", frame)
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break

    cap.release()
    cv2.destroyAllWindows()

if __name__ == '__main__':
    main()
